#!/bin/bash

JuceVersion="7.0.5"
LLVMVersion="11"
BoostVersion="1.84.0"

set -e

if [ "$EUID" -ne 0 ]
  then echo "Please run as root"
  exit
fi

echo "Installing Prerequisites"
apt-get install curl tar gzip build-essential -y
apt-get install libasound2-dev libfreetype6-dev libfontconfig1-dev xclip libcurl4-openssl-dev  -y
apt-get install webkit2gtk-4.0 libgtk-3-dev -y 
apt-get install xorg-dev -y

echo "Installing CMake"
apt-get install cmake ninja-build pkg-config -y

echo "Installing LLVM"
apt-get install llvm-$LLVMVersion clang-$LLVMVersion -y

echo "Installing Boost"
BoostVersion_=$(echo boost_$BoostVersion | tr '.' '_')
wget "https://boostorg.jfrog.io/artifactory/main/release/$BoostVersion/source/boost_$(echo $BoostVersion | tr '.' '_').tar.gz"
tar xzfv "$BoostVersion_.tar.gz"
cd "$BoostVersion_"
./bootstrap.sh --prefix=/usr/local/
./b2 link=static runtime-link=static --with-date_time	--with-filesystem --with-iostreams
./b2 install
cd ..

echo "Installing JUCE"
if [[ ! -d "JUCE" ]]; then
	curl -s -L "https://github.com/juce-framework/JUCE/archive/refs/tags/$JuceVersion.tar.gz" | tar xvz -C .
	mv JUCE-$JuceVersion JUCE
  chmod -R 777 JUCE
fi

echo "Installing cpprestsdk"
apt-get install libcpprest-dev -y